@@ -3,6 +3,7 @@ import os
 import re
 import signal
 import sys
+import threading
 import time
 from asyncio import (
     AbstractEventLoop,
@@ -41,6 +42,7 @@ from typing import (
 from prompt_toolkit.buffer import Buffer
 from prompt_toolkit.cache import SimpleCache
 from prompt_toolkit.clipboard import Clipboard, InMemoryClipboard
+from prompt_toolkit.data_structures import Size
 from prompt_toolkit.enums import EditingMode
 from prompt_toolkit.eventloop import (
     get_traceback_from_context,
@@ -119,8 +121,6 @@ class Application(Generic[_AppResult]):
         :class:`~prompt_toolkit.key_binding.KeyBindingsBase` instance for
         the key bindings.
     :param clipboard: :class:`~prompt_toolkit.clipboard.Clipboard` to use.
-    :param on_abort: What to do when Control-C is pressed.
-    :param on_exit: What to do when Control-D is pressed.
     :param full_screen: When True, run the application on the alternate screen buffer.
     :param color_depth: Any :class:`~.ColorDepth` value, a callable that
         returns a :class:`~.ColorDepth` or `None` for default.
@@ -147,6 +147,10 @@ class Application(Generic[_AppResult]):
         seconds. When `None` (the default), only invalidate when `invalidate`
         has been called.
 
+    :param terminal_size_polling_interval: Poll the terminal size every so many
+        seconds. Useful if the applications runs in a thread other then then
+        main thread where SIGWINCH can't be handled, or on Windows.
+
     Filters:
 
     :param mouse_support: (:class:`~prompt_toolkit.filters.Filter` or
@@ -211,6 +215,7 @@ class Application(Generic[_AppResult]):
         min_redraw_interval: Union[float, int, None] = None,
         max_render_postpone_time: Union[float, int, None] = 0.01,
         refresh_interval: Optional[float] = None,
+        terminal_size_polling_interval: Optional[float] = 0.5,
         on_reset: Optional[ApplicationEventHandler] = None,
         on_invalidate: Optional[ApplicationEventHandler] = None,
         before_render: Optional[ApplicationEventHandler] = None,
@@ -218,7 +223,7 @@ class Application(Generic[_AppResult]):
         # I/O.
         input: Optional[Input] = None,
         output: Optional[Output] = None,
-    ):
+    ) -> None:
 
         # If `enable_page_navigation_bindings` is not specified, enable it in
         # case of full screen applications only. This can be overridden by the user.
@@ -259,6 +264,7 @@ class Application(Generic[_AppResult]):
         self.min_redraw_interval = min_redraw_interval
         self.max_render_postpone_time = max_render_postpone_time
         self.refresh_interval = refresh_interval
+        self.terminal_size_polling_interval = terminal_size_polling_interval
 
         # Events.
         self.on_invalidate = Event(self, on_invalidate)
@@ -368,7 +374,8 @@ class Application(Generic[_AppResult]):
         The active :class:`.ColorDepth`.
 
         The current value is determined as follows:
-        - If a color depth was given explicitely to this application, use that
+
+        - If a color depth was given explicitly to this application, use that
           value.
         - Otherwise, fall back to the color depth that is reported by the
           :class:`.Output` implementation. If the :class:`.Output` class was
@@ -452,6 +459,11 @@ class Application(Generic[_AppResult]):
             # See: https://github.com/dbcli/mycli/issues/797
             return
 
+        # `invalidate()` called if we don't have a loop yet (not running?), or
+        # after the event loop was closed.
+        if self.loop is None or self.loop.is_closed():
+            return
+
         # Never schedule a second redraw, when a previous one has not yet been
         # executed. (This should protect against other threads calling
         # 'invalidate' many times, resulting in 100% CPU.)
@@ -461,7 +473,7 @@ class Application(Generic[_AppResult]):
             self._invalidated = True
 
         # Trigger event.
-        self.on_invalidate.fire()
+        self.loop.call_soon_threadsafe(self.on_invalidate.fire)
 
         def redraw() -> None:
             self._invalidated = False
@@ -482,7 +494,9 @@ class Application(Generic[_AppResult]):
                     await sleep(cast(float, self.min_redraw_interval) - diff)
                     schedule_redraw()
 
-                self.create_background_task(redraw_in_future())
+                self.loop.call_soon_threadsafe(
+                    lambda: self.create_background_task(redraw_in_future())
+                )
             else:
                 schedule_redraw()
         else:
@@ -490,7 +504,7 @@ class Application(Generic[_AppResult]):
 
     @property
     def invalidated(self) -> bool:
-        " True when a redraw operation has been scheduled. "
+        "True when a redraw operation has been scheduled."
         return self._invalidated
 
     def _redraw(self, render_as_done: bool = False) -> None:
@@ -531,17 +545,20 @@ class Application(Generic[_AppResult]):
         #       invalidate function is often called from a context where this
         #       application is not the active one. (Like the
         #       `PromptSession._auto_refresh_context`).
+        #       We copy the context in case the context was already active, to
+        #       prevent RuntimeErrors. (The rendering is not supposed to change
+        #       any context variables.)
         if self.context is not None:
-            self.context.run(run_in_context)
+            self.context.copy().run(run_in_context)
 
     def _start_auto_refresh_task(self) -> None:
         """
         Start a while/true loop in the background for automatic invalidation of
         the UI.
         """
-        if self.refresh_interval not in (None, 0):
+        if self.refresh_interval is not None and self.refresh_interval != 0:
 
-            async def auto_refresh(refresh_interval) -> None:
+            async def auto_refresh(refresh_interval: float) -> None:
                 while True:
                     await sleep(refresh_interval)
                     self.invalidate()
@@ -593,7 +610,15 @@ class Application(Generic[_AppResult]):
         self._redraw()
 
     def _pre_run(self, pre_run: Optional[Callable[[], None]] = None) -> None:
-        " Called during `run`. "
+        """
+        Called during `run`.
+
+        `self.future` should be set to the new future at the point where this
+        is called in order to avoid data races. `pre_run` can be used to set a
+        `threading.Event` to synchronize with UI termination code, running in
+        another thread that would call `Application.exit`. (See the progress
+        bar code for an example.)
+        """
         if pre_run:
             pre_run()
 
@@ -626,7 +651,7 @@ class Application(Generic[_AppResult]):
         assert not self._is_running, "Application is already running."
 
         async def _run_async() -> _AppResult:
-            " Coroutine. "
+            "Coroutine."
             loop = get_event_loop()
             f = loop.create_future()
             self.future = f  # XXX: make sure to set this before calling '_redraw'.
@@ -640,6 +665,7 @@ class Application(Generic[_AppResult]):
             flush_task: Optional[asyncio.Task[None]] = None
 
             # Reset.
+            # (`self.future` needs to be set when `pre_run` is called.)
             self.reset()
             self._pre_run(pre_run)
 
@@ -666,7 +692,8 @@ class Application(Generic[_AppResult]):
 
                 # Quit when the input stream was closed.
                 if self.input.closed:
-                    f.set_exception(EOFError)
+                    if not f.done():
+                        f.set_exception(EOFError)
                 else:
                     # Automatically flush keys.
                     if flush_task:
@@ -694,6 +721,8 @@ class Application(Generic[_AppResult]):
             with self.input.raw_mode(), self.input.attach(
                 read_from_input
             ), attach_winch_signal_handler(self._on_resize):
+                self.create_background_task(self._poll_output_size())
+
                 # Draw UI.
                 self._request_absolute_cursor_position()
                 self._redraw()
@@ -773,6 +802,12 @@ class Application(Generic[_AppResult]):
                         # above, but in case of exceptions, that's not always the
                         # case.
                         self._is_running = False
+
+                        # Also remove the Future again. (This brings the
+                        # application back to its initial state, where it also
+                        # doesn't have a Future.)
+                        self.future = None
+
                     return result
             finally:
                 if set_exception_handler:
@@ -784,19 +819,61 @@ class Application(Generic[_AppResult]):
         self,
         pre_run: Optional[Callable[[], None]] = None,
         set_exception_handler: bool = True,
+        in_thread: bool = False,
     ) -> _AppResult:
         """
         A blocking 'run' call that waits until the UI is finished.
 
         This will start the current asyncio event loop. If no loop is set for
-        the current thread, then it will create a new loop.
+        the current thread, then it will create a new loop. If a new loop was
+        created, this won't close the new loop (if `in_thread=False`).
 
         :param pre_run: Optional callable, which is called right after the
             "reset" of the application.
         :param set_exception_handler: When set, in case of an exception, go out
             of the alternate screen and hide the application, display the
             exception, and wait for the user to press ENTER.
+        :param in_thread: When true, run the application in a background
+            thread, and block the current thread until the application
+            terminates. This is useful if we need to be sure the application
+            won't use the current event loop (asyncio does not support nested
+            event loops). A new event loop will be created in this background
+            thread, and that loop will also be closed when the background
+            thread terminates. When this is used, it's especially important to
+            make sure that all asyncio background tasks are managed through
+            `get_appp().create_background_task()`, so that unfinished tasks are
+            properly cancelled before the event loop is closed. This is used
+            for instance in ptpython.
         """
+        if in_thread:
+            result: _AppResult
+            exception: Optional[BaseException] = None
+
+            def run_in_thread() -> None:
+                nonlocal result, exception
+                try:
+                    result = self.run(
+                        pre_run=pre_run, set_exception_handler=set_exception_handler
+                    )
+                except BaseException as e:
+                    exception = e
+                finally:
+                    # Make sure to close the event loop in this thread. Running
+                    # the application creates a new loop (because we're in
+                    # another thread), but it doesn't get closed automatically
+                    # (also not by the garbage collector).
+                    loop = get_event_loop()
+                    loop.run_until_complete(loop.shutdown_asyncgens())
+                    loop.close()
+
+            thread = threading.Thread(target=run_in_thread)
+            thread.start()
+            thread.join()
+
+            if exception is not None:
+                raise exception
+            return result
+
         # We don't create a new event loop by default, because we want to be
         # sure that when this is called multiple times, each call of `run()`
         # goes through the same event loop. This way, users can schedule
@@ -845,9 +922,14 @@ class Application(Generic[_AppResult]):
         self, coroutine: Awaitable[None]
     ) -> "asyncio.Task[None]":
         """
-        Start a background task (coroutine) for the running application.
+        Start a background task (coroutine) for the running application. When
+        the `Application` terminates, unfinished background tasks will be
+        cancelled.
+
         If asyncio had nurseries like Trio, we would create a nursery in
         `Application.run_async`, and run the given coroutine in that nursery.
+
+        Not threadsafe.
         """
         task = get_event_loop().create_task(coroutine)
         self.background_tasks.append(task)
@@ -871,6 +953,28 @@ class Application(Generic[_AppResult]):
             except CancelledError:
                 pass
 
+    async def _poll_output_size(self) -> None:
+        """
+        Coroutine for polling the terminal dimensions.
+
+        Useful for situations where `attach_winch_signal_handler` is not sufficient:
+        - If we are not running in the main thread.
+        - On Windows.
+        """
+        size: Optional[Size] = None
+        interval = self.terminal_size_polling_interval
+
+        if interval is None:
+            return
+
+        while True:
+            await asyncio.sleep(interval)
+            new_size = self.output.get_size()
+
+            if size is not None and new_size != size:
+                self._on_resize()
+            size = new_size
+
     def cpr_not_supported_callback(self) -> None:
         """
         Called when we don't receive the cursor position response in time.
@@ -888,17 +992,17 @@ class Application(Generic[_AppResult]):
 
     @overload
     def exit(self) -> None:
-        " Exit without arguments. "
+        "Exit without arguments."
 
     @overload
     def exit(self, *, result: _AppResult, style: str = "") -> None:
-        " Exit with `_AppResult`. "
+        "Exit with `_AppResult`."
 
     @overload
     def exit(
         self, *, exception: Union[BaseException, Type[BaseException]], style: str = ""
     ) -> None:
-        " Exit with exception. "
+        "Exit with exception."
 
     def exit(
         self,
@@ -909,6 +1013,14 @@ class Application(Generic[_AppResult]):
         """
         Exit application.
 
+        .. note::
+
+            If `Application.exit` is called before `Application.run()` is
+            called, then the `Application` won't exit (because the
+            `Application.future` doesn't correspond to the current run). Use a
+            `pre_run` hook and an event to synchronize the closing if there's a
+            chance this can happen.
+
         :param result: Set this result for the application.
         :param exception: Set this exception as the result for an application. For
             a prompt, this is often `EOFError` or `KeyboardInterrupt`.
@@ -1029,7 +1141,7 @@ class Application(Generic[_AppResult]):
 
     @property
     def is_running(self) -> bool:
-        " `True` when the application is currently active/running. "
+        "`True` when the application is currently active/running."
         return self._is_running
 
     @property
@@ -1168,7 +1280,7 @@ async def _do_wait_for_enter(wait_text: AnyFormattedText) -> None:
 
     @key_bindings.add(Keys.Any)
     def _ignore(event: E) -> None:
-        " Disallow typing. "
+        "Disallow typing."
         pass
 
     session: PromptSession[None] = PromptSession(
