@@ -61,11 +61,11 @@ logger = logging.getLogger(__name__)
 
 
 class EditReadOnlyBuffer(Exception):
-    " Attempt editing of read-only :class:`.Buffer`. "
+    "Attempt editing of read-only :class:`.Buffer`."
 
 
 class ValidationState(Enum):
-    " The validation state of a buffer. This is set after the validation. "
+    "The validation state of a buffer. This is set after the validation."
     VALID = "VALID"
     INVALID = "INVALID"
     UNKNOWN = "UNKNOWN"
@@ -202,11 +202,11 @@ class Buffer:
 
     Events:
 
-    :param on_text_changed: When the buffer text changes. (Callable on None.)
-    :param on_text_insert: When new text is inserted. (Callable on None.)
-    :param on_cursor_position_changed: When the cursor moves. (Callable on None.)
-    :param on_completions_changed: When the completions were changed. (Callable on None.)
-    :param on_suggestion_set: When an auto-suggestion text has been set. (Callable on None.)
+    :param on_text_changed: When the buffer text changes. (Callable or None.)
+    :param on_text_insert: When new text is inserted. (Callable or None.)
+    :param on_cursor_position_changed: When the cursor moves. (Callable or None.)
+    :param on_completions_changed: When the completions were changed. (Callable or None.)
+    :param on_suggestion_set: When an auto-suggestion text has been set. (Callable or None.)
 
     Filters:
 
@@ -408,23 +408,39 @@ class Buffer:
         if self._load_history_task is None:
 
             async def load_history() -> None:
+                async for item in self.history.load():
+                    self._working_lines.appendleft(item)
+                    self.__working_index += 1
+
+            self._load_history_task = get_app().create_background_task(load_history())
+
+            def load_history_done(f: "asyncio.Future[None]") -> None:
+                """
+                Handle `load_history` result when either done, cancelled, or
+                when an exception was raised.
+                """
                 try:
-                    async for item in self.history.load():
-                        self._working_lines.appendleft(item)
-                        self.__working_index += 1
+                    f.result()
                 except asyncio.CancelledError:
+                    # Ignore cancellation. But handle it, so that we don't get
+                    # this traceback.
+                    pass
+                except GeneratorExit:
+                    # Probably not needed, but we had situations where
+                    # `GeneratorExit` was raised in `load_history` during
+                    # cancellation.
                     pass
                 except BaseException:
                     # Log error if something goes wrong. (We don't have a
                     # caller to which we can propagate this exception.)
                     logger.exception("Loading history failed")
 
-            self._load_history_task = asyncio.ensure_future(load_history())
+            self._load_history_task.add_done_callback(load_history_done)
 
     # <getters/setters>
 
     def _set_text(self, value: str) -> bool:
-        """ set text at current working_index. Return whether it changed. """
+        """set text at current working_index. Return whether it changed."""
         working_index = self.working_index
         working_lines = self._working_lines
 
@@ -444,11 +460,11 @@ class Buffer:
         return False
 
     def _set_cursor_position(self, value: int) -> bool:
-        """ Set cursor position. Return whether it changed. """
+        """Set cursor position. Return whether it changed."""
         original_position = self.__cursor_position
         self.__cursor_position = max(0, value)
 
-        return value != original_position
+        return self.__cursor_position != original_position
 
     @property
     def text(self) -> str:
@@ -709,7 +725,7 @@ class Buffer:
         self.cursor_position += self.document.get_cursor_right_position(count=count)
 
     def cursor_up(self, count: int = 1) -> None:
-        """ (for multiline edit). Move cursor to the previous line.  """
+        """(for multiline edit). Move cursor to the previous line."""
         original_column = self.preferred_column or self.document.cursor_position_col
         self.cursor_position += self.document.get_cursor_up_position(
             count=count, preferred_column=original_column
@@ -719,7 +735,7 @@ class Buffer:
         self.preferred_column = original_column
 
     def cursor_down(self, count: int = 1) -> None:
-        """ (for multiline edit). Move cursor to the next line.  """
+        """(for multiline edit). Move cursor to the next line."""
         original_column = self.preferred_column or self.document.cursor_position_col
         self.cursor_position += self.document.get_cursor_down_position(
             count=count, preferred_column=original_column
@@ -1864,7 +1880,7 @@ class Buffer:
                 self.reset()
 
 
-_T = TypeVar("_T", bound=Callable[..., Awaitable])
+_T = TypeVar("_T", bound=Callable[..., Awaitable[None]])
 
 
 def _only_one_at_a_time(coroutine: _T) -> _T:
@@ -1902,7 +1918,7 @@ def _only_one_at_a_time(coroutine: _T) -> _T:
 
 
 class _Retry(Exception):
-    " Retry in `_only_one_at_a_time`. "
+    "Retry in `_only_one_at_a_time`."
 
 
 def indent(buffer: Buffer, from_row: int, to_row: int, count: int = 1) -> None:
