@@ -1,12 +1,29 @@
-import msvcrt
 import os
 import sys
 from abc import abstractmethod
 from asyncio import get_event_loop
 from contextlib import contextmanager
-from ctypes import pointer, windll
+
+from ..utils import SPHINX_AUTODOC_RUNNING
+
+# Do not import win32-specific stuff when generating documentation.
+# Otherwise RTD would be unable to generate docs for this module.
+if not SPHINX_AUTODOC_RUNNING:
+    import msvcrt
+    from ctypes import windll
+
+from ctypes import pointer
 from ctypes.wintypes import DWORD, HANDLE
-from typing import Callable, ContextManager, Dict, Iterable, Optional, TextIO
+from typing import (
+    Callable,
+    ContextManager,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    TextIO,
+)
 
 from prompt_toolkit.eventloop import run_in_executor_with_context
 from prompt_toolkit.eventloop.win32 import create_win32_event, wait_for_handles
@@ -71,7 +88,7 @@ class Win32Input(_Win32InputBase):
         """
         return detach_win32_input(self)
 
-    def read_keys(self):
+    def read_keys(self) -> List[KeyPress]:
         return list(self.console_input_reader.read())
 
     def flush(self) -> None:
@@ -197,7 +214,7 @@ class ConsoleInputReader:
             self.handle = HANDLE(msvcrt.get_osfhandle(self._fdcon))
 
     def close(self) -> None:
-        " Close fdcon. "
+        "Close fdcon."
         if self._fdcon is not None:
             os.close(self._fdcon)
 
@@ -272,7 +289,7 @@ class ConsoleInputReader:
 
         return KeyPress(key_press.key, data)
 
-    def _get_keys(self, read, input_records):
+    def _get_keys(self, read: DWORD, input_records) -> Iterator[KeyPress]:
         """
         Generator that yields `KeyPress` objects from the input records.
         """
@@ -297,7 +314,7 @@ class ConsoleInputReader:
                         yield key_press
 
     @staticmethod
-    def _is_paste(keys) -> bool:
+    def _is_paste(keys: List[KeyPress]) -> bool:
         """
         Return `True` when we should consider this list of keys as a paste
         event. Pasted text on windows will be turned into a
@@ -318,13 +335,13 @@ class ConsoleInputReader:
 
         return newline_count >= 1 and text_count > 1
 
-    def _event_to_key_presses(self, ev):
+    def _event_to_key_presses(self, ev: KEY_EVENT_RECORD) -> List[KeyPress]:
         """
         For this `KEY_EVENT_RECORD`, return a list of `KeyPress` instances.
         """
         assert type(ev) == KEY_EVENT_RECORD and ev.KeyDown
 
-        result = None
+        result: Optional[KeyPress] = None
 
         u_char = ev.uChar.UnicodeChar
         ascii_char = u_char.encode("utf-8")
@@ -356,70 +373,55 @@ class ConsoleInputReader:
             and ev.ControlKeyState & self.SHIFT_PRESSED
             and result
         ):
-            if result.key == Keys.Left:
-                result.key = Keys.ShiftControlLeft
+            mapping: Dict[str, str] = {
+                Keys.Left: Keys.ControlShiftLeft,
+                Keys.Right: Keys.ControlShiftRight,
+                Keys.Up: Keys.ControlShiftUp,
+                Keys.Down: Keys.ControlShiftDown,
+                Keys.Home: Keys.ControlShiftHome,
+                Keys.End: Keys.ControlShiftEnd,
+                Keys.Insert: Keys.ControlShiftInsert,
+                Keys.PageUp: Keys.ControlShiftPageUp,
+                Keys.PageDown: Keys.ControlShiftPageDown,
+            }
+            result.key = mapping.get(result.key, result.key)
 
-            if result.key == Keys.Right:
-                result.key = Keys.ShiftControlRight
-
-            if result.key == Keys.Home:
-                result.key = Keys.ShiftControlHome
-
-            if result.key == Keys.End:
-                result.key = Keys.ShiftControlEnd
-
-        # Correctly handle Control-Arrow/Home/End and Control-Insert keys.
+        # Correctly handle Control-Arrow/Home/End and Control-Insert/Delete keys.
         if (
             ev.ControlKeyState & self.LEFT_CTRL_PRESSED
             or ev.ControlKeyState & self.RIGHT_CTRL_PRESSED
         ) and result:
-            if result.key == Keys.Left:
-                result.key = Keys.ControlLeft
-
-            if result.key == Keys.Right:
-                result.key = Keys.ControlRight
-
-            if result.key == Keys.Up:
-                result.key = Keys.ControlUp
-
-            if result.key == Keys.Down:
-                result.key = Keys.ControlDown
-
-            if result.key == Keys.Home:
-                result.key = Keys.ControlHome
-
-            if result.key == Keys.End:
-                result.key = Keys.ControlEnd
-
-            if result.key == Keys.Insert:
-                result.key = Keys.ControlInsert
+            mapping = {
+                Keys.Left: Keys.ControlLeft,
+                Keys.Right: Keys.ControlRight,
+                Keys.Up: Keys.ControlUp,
+                Keys.Down: Keys.ControlDown,
+                Keys.Home: Keys.ControlHome,
+                Keys.End: Keys.ControlEnd,
+                Keys.Insert: Keys.ControlInsert,
+                Keys.Delete: Keys.ControlDelete,
+                Keys.PageUp: Keys.ControlPageUp,
+                Keys.PageDown: Keys.ControlPageDown,
+            }
+            result.key = mapping.get(result.key, result.key)
 
         # Turn 'Tab' into 'BackTab' when shift was pressed.
         # Also handle other shift-key combination
         if ev.ControlKeyState & self.SHIFT_PRESSED and result:
-            if result.key == Keys.Tab:
-                result.key = Keys.BackTab
-
-            if result.key == Keys.Left:
-                result.key = Keys.ShiftLeft
-
-            if result.key == Keys.Right:
-                result.key = Keys.ShiftRight
-
-            if result.key == Keys.Up:
-                result.key = Keys.ShiftUp
-
-            if result.key == Keys.Down:
-                result.key = Keys.ShiftDown
-
-            if result.key == Keys.Home:
-                result.key = Keys.ShiftHome
-
-            if result.key == Keys.End:
-                result.key = Keys.ShiftEnd
-
-            if result.key == Keys.Insert:
-                result.key = Keys.ShiftInsert
+            mapping = {
+                Keys.Tab: Keys.BackTab,
+                Keys.Left: Keys.ShiftLeft,
+                Keys.Right: Keys.ShiftRight,
+                Keys.Up: Keys.ShiftUp,
+                Keys.Down: Keys.ShiftDown,
+                Keys.Home: Keys.ShiftHome,
+                Keys.End: Keys.ShiftEnd,
+                Keys.Insert: Keys.ShiftInsert,
+                Keys.Delete: Keys.ShiftDelete,
+                Keys.PageUp: Keys.ShiftPageUp,
+                Keys.PageDown: Keys.ShiftPageDown,
+            }
+            result.key = mapping.get(result.key, result.key)
 
         # Turn 'Space' into 'ControlSpace' when control was pressed.
         if (
@@ -463,23 +465,42 @@ class ConsoleInputReader:
         else:
             return []
 
-    def _handle_mouse(self, ev):
+    def _handle_mouse(self, ev: MOUSE_EVENT_RECORD) -> List[KeyPress]:
         """
         Handle mouse events. Return a list of KeyPress instances.
         """
         FROM_LEFT_1ST_BUTTON_PRESSED = 0x1
+        MOUSE_MOVED = 0x0001
+        MOUSE_WHEELED = 0x0004
 
         result = []
+        event_type: Optional[MouseEventType] = None
 
-        # Check event type.
-        if ev.ButtonState == FROM_LEFT_1ST_BUTTON_PRESSED:
-            # On a key press, generate both the mouse down and up event.
-            for event_type in [MouseEventType.MOUSE_DOWN, MouseEventType.MOUSE_UP]:
-                data = ";".join(
-                    [str(event_type), str(ev.MousePosition.X), str(ev.MousePosition.Y)]
-                )
-                result.append(KeyPress(Keys.WindowsMouseEvent, data))
+        # Move events.
+        if ev.EventFlags & MOUSE_MOVED:
+            if ev.ButtonState == FROM_LEFT_1ST_BUTTON_PRESSED:
+                event_type = MouseEventType.MOUSE_DOWN_MOVE
 
+        # Scroll events.
+        elif ev.EventFlags & MOUSE_WHEELED:
+            if ev.ButtonState > 0:
+                event_type = MouseEventType.SCROLL_UP
+            else:
+                event_type = MouseEventType.SCROLL_DOWN
+
+        # Mouse down (left button).
+        elif ev.ButtonState == FROM_LEFT_1ST_BUTTON_PRESSED:
+            event_type = MouseEventType.MOUSE_DOWN
+
+        # No key pressed anymore: mouse up.
+        else:
+            event_type = MouseEventType.MOUSE_UP
+
+        if event_type is not None:
+            data = ";".join(
+                [event_type.value, str(ev.MousePosition.X), str(ev.MousePosition.Y)]
+            )
+            result.append(KeyPress(Keys.WindowsMouseEvent, data))
         return result
 
 
@@ -626,10 +647,10 @@ class raw_mode:
     `raw_input` method of `.vt100_input`.
     """
 
-    def __init__(self, fileno=None):
+    def __init__(self, fileno: Optional[int] = None) -> None:
         self.handle = HANDLE(windll.kernel32.GetStdHandle(STD_INPUT_HANDLE))
 
-    def __enter__(self):
+    def __enter__(self) -> None:
         # Remember original mode.
         original_mode = DWORD()
         windll.kernel32.GetConsoleMode(self.handle, pointer(original_mode))
