@@ -171,14 +171,14 @@ AnyContainer = Union[Container, "MagicContainer"]
 
 
 def _window_too_small() -> "Window":
-    " Create a `Window` that displays the 'Window too small' text. "
+    "Create a `Window` that displays the 'Window too small' text."
     return Window(
         FormattedTextControl(text=[("class:window-too-small", " Window too small... ")])
     )
 
 
 class VerticalAlign(Enum):
-    " Alignment for `HSplit`. "
+    "Alignment for `HSplit`."
     TOP = "TOP"
     CENTER = "CENTER"
     BOTTOM = "BOTTOM"
@@ -186,7 +186,7 @@ class VerticalAlign(Enum):
 
 
 class HorizontalAlign(Enum):
-    " Alignment for `VSplit`. "
+    "Alignment for `VSplit`."
     LEFT = "LEFT"
     CENTER = "CENTER"
     RIGHT = "RIGHT"
@@ -751,7 +751,7 @@ class FloatContainer(Container):
                        floats=[
                            Float(xcursor=True,
                                 ycursor=True,
-                                layout=CompletionMenu(...))
+                                content=CompletionsMenu(...))
                        ])
 
     :param z_index: (int or None) When specified, this can be used to bring
@@ -861,7 +861,7 @@ class FloatContainer(Container):
         erase_bg: bool,
         z_index: Optional[int],
     ) -> None:
-        " Draw a single Float. "
+        "Draw a single Float."
         # When a menu_position was given, use this instead of the cursor
         # position. (These cursor positions are absolute, translate again
         # relative to the write_position.)
@@ -1067,7 +1067,7 @@ class Float:
         allow_cover_cursor: bool = False,
         z_index: int = 1,
         transparent: bool = False,
-    ):
+    ) -> None:
 
         assert z_index >= 1
 
@@ -1687,7 +1687,7 @@ class Window(Container):
         return self._ui_content_cache.get(key, get_content)
 
     def _get_digraph_char(self) -> Optional[str]:
-        " Return `False`, or the Digraph symbol to be used. "
+        "Return `False`, or the Digraph symbol to be used."
         app = get_app()
         if app.quoted_insert:
             return "^"
@@ -1891,7 +1891,7 @@ class Window(Container):
         move_x = 0
 
         def render_margin(m: Margin, width: int) -> UIContent:
-            " Render margin. Return `Screen`. "
+            "Render margin. Return `Screen`."
             # Retrieve margin fragments.
             fragments = m.create_margin(render_info, width, write_position.height)
 
@@ -2110,7 +2110,7 @@ class Window(Container):
         copy()
 
         def cursor_pos_to_screen_pos(row: int, col: int) -> Point:
-            " Translate row/col from UIContent to real Screen coordinates. "
+            "Translate row/col from UIContent to real Screen coordinates."
             try:
                 y, x = rowcol_to_yx[row, col]
             except KeyError:
@@ -2474,7 +2474,7 @@ class Window(Container):
             window_size: int,
             content_size: int,
         ) -> int:
-            " Scrolling algorithm. Used for both horizontal and vertical scrolling. "
+            "Scrolling algorithm. Used for both horizontal and vertical scrolling."
             # Calculate the scroll offset to apply.
             # This can obviously never be more than have the screen size. Also, when the
             # cursor appears at the top or bottom, we don't apply the offset.
@@ -2558,7 +2558,7 @@ class Window(Container):
             self._scroll_up()
 
     def _scroll_down(self) -> None:
-        " Scroll window down. "
+        "Scroll window down."
         info = self.render_info
 
         if info is None:
@@ -2571,7 +2571,7 @@ class Window(Container):
             self.vertical_scroll += 1
 
     def _scroll_up(self) -> None:
-        " Scroll window up. "
+        "Scroll window up."
         info = self.render_info
 
         if info is None:
@@ -2674,8 +2674,18 @@ class DynamicContainer(Container):
     def preferred_height(self, width: int, max_available_height: int) -> Dimension:
         return self._get_container().preferred_height(width, max_available_height)
 
-    def write_to_screen(self, *a, **kw):
-        self._get_container().write_to_screen(*a, **kw)
+    def write_to_screen(
+        self,
+        screen: Screen,
+        mouse_handlers: MouseHandlers,
+        write_position: WritePosition,
+        parent_style: str,
+        erase_bg: bool,
+        z_index: Optional[int],
+    ) -> None:
+        self._get_container().write_to_screen(
+            screen, mouse_handlers, write_position, parent_style, erase_bg, z_index
+        )
 
     def is_modal(self) -> bool:
         return False
@@ -2700,7 +2710,7 @@ def to_container(container: AnyContainer) -> Container:
     if isinstance(container, Container):
         return container
     elif hasattr(container, "__pt_container__"):
-        return to_container(cast("MagicContainer", container).__pt_container__())
+        return to_container(container.__pt_container__())
     else:
         raise ValueError("Not a container object: %r" % (container,))
 
