@@ -39,7 +39,7 @@ class Dimension:
         max: Optional[int] = None,
         weight: Optional[int] = None,
         preferred: Optional[int] = None,
-    ):
+    ) -> None:
         if weight is not None:
             assert weight >= 0  # Also cannot be a float.
 
@@ -94,7 +94,7 @@ class Dimension:
         return cls.exact(amount=0)
 
     def is_zero(self) -> bool:
-        " True if this `Dimension` represents a zero size. "
+        "True if this `Dimension` represents a zero size."
         return self.preferred == 0 or self.max == 0
 
     def __repr__(self) -> str:
