@@ -84,13 +84,13 @@ def _output_screen_diff(
     output.hide_cursor()
 
     def reset_attributes() -> None:
-        " Wrapper around Output.reset_attributes. "
+        "Wrapper around Output.reset_attributes."
         nonlocal last_style
         _output_reset_attributes()
         last_style = None  # Forget last char after resetting attributes.
 
     def move_cursor(new: Point) -> Point:
-        " Move cursor to this `new` point. Returns the given Point. "
+        "Move cursor to this `new` point. Returns the given Point."
         current_x, current_y = current_pos.x, current_pos.y
 
         if new.y > current_y:
@@ -262,7 +262,7 @@ def _output_screen_diff(
 
 
 class HeightIsUnknownError(Exception):
-    " Information unavailable. Did not yet receive the CPR response. "
+    "Information unavailable. Did not yet receive the CPR response."
 
 
 class _StyleStringToAttrsCache(Dict[str, Attrs]):
@@ -317,7 +317,7 @@ class _StyleStringHasStyleCache(Dict[str, bool]):
 
 
 class CPR_Support(Enum):
-    " Enum: whether or not CPR is supported. "
+    "Enum: whether or not CPR is supported."
     SUPPORTED = "SUPPORTED"
     NOT_SUPPORTED = "NOT_SUPPORTED"
     UNKNOWN = "UNKNOWN"
@@ -354,6 +354,7 @@ class Renderer:
         self._in_alternate_screen = False
         self._mouse_support_enabled = False
         self._bracketed_paste_enabled = False
+        self._cursor_key_mode_reset = False
 
         # Future set when we are waiting for a CPR flag.
         self._waiting_for_cpr_futures: Deque[Future[None]] = deque()
@@ -412,6 +413,8 @@ class Renderer:
             self.output.disable_bracketed_paste()
             self._bracketed_paste_enabled = False
 
+        # NOTE: No need to set/reset cursor key mode here.
+
         # Flush output. `disable_mouse_support` needs to write to stdout.
         self.output.flush()
 
@@ -595,6 +598,11 @@ class Renderer:
             self.output.enable_bracketed_paste()
             self._bracketed_paste_enabled = True
 
+        # Reset cursor key mode.
+        if not self._cursor_key_mode_reset:
+            self.output.reset_cursor_key_mode()
+            self._cursor_key_mode_reset = True
+
         # Enable/disable mouse support.
         needs_mouse_support = self.mouse_support()
 
