@@ -25,12 +25,14 @@ Example::
         result = s.prompt('Say something: ')
 """
 from asyncio import get_event_loop
+from contextlib import contextmanager
 from enum import Enum
 from functools import partial
 from typing import (
     TYPE_CHECKING,
     Callable,
     Generic,
+    Iterator,
     List,
     Optional,
     Tuple,
@@ -591,7 +593,7 @@ class PromptSession(Generic[_T]):
         )
 
         def get_search_buffer_control() -> SearchBufferControl:
-            " Return the UIControl to be focused when searching start. "
+            "Return the UIControl to be focused when searching start."
             if is_true(self.multiline):
                 return search_toolbar.control
             else:
@@ -792,7 +794,7 @@ class PromptSession(Generic[_T]):
 
         @handle("enter", filter=do_accept & default_focused)
         def _accept_input(event: E) -> None:
-            " Accept input when enter has been pressed. "
+            "Accept input when enter has been pressed."
             self.default_buffer.validate_and_handle()
 
         @Condition
@@ -801,12 +803,12 @@ class PromptSession(Generic[_T]):
 
         @handle("tab", filter=readline_complete_style & default_focused)
         def _complete_like_readline(event: E) -> None:
-            " Display completions (like Readline). "
+            "Display completions (like Readline)."
             display_completions_like_readline(event)
 
         @handle("c-c", filter=default_focused)
         def _keyboard_interrupt(event: E) -> None:
-            " Abort when Control-C has been pressed. "
+            "Abort when Control-C has been pressed."
             event.app.exit(exception=KeyboardInterrupt, style="class:aborting")
 
         @Condition
@@ -821,7 +823,7 @@ class PromptSession(Generic[_T]):
 
         @handle("c-d", filter=ctrl_d_condition & default_focused)
         def _eof(event: E) -> None:
-            " Exit when Control-D has been pressed. "
+            "Exit when Control-D has been pressed."
             event.app.exit(exception=EOFError, style="class:exiting")
 
         suspend_supported = Condition(suspend_to_background_supported)
@@ -887,6 +889,7 @@ class PromptSession(Generic[_T]):
         accept_default: bool = False,
         pre_run: Optional[Callable[[], None]] = None,
         set_exception_handler: bool = True,
+        in_thread: bool = False,
     ) -> _T:
         """
         Display the prompt.
@@ -909,6 +912,9 @@ class PromptSession(Generic[_T]):
         :param accept_default: When `True`, automatically accept the default
             value without allowing the user to edit the input.
         :param pre_run: Callable, called at the start of `Application.run`.
+        :param in_thread: Run the prompt in a background thread; block the
+            current thread. This avoids interference with an event loop in the
+            current thread. Like `Application.run(in_thread=True)`.
 
         This method will raise ``KeyboardInterrupt`` when control-c has been
         pressed (for abort) and ``EOFError`` when control-d has been pressed
@@ -1008,13 +1014,17 @@ class PromptSession(Generic[_T]):
         # If we are using the default output, and have a dumb terminal. Use the
         # dumb prompt.
         if self._output is None and is_dumb_terminal():
-            return get_event_loop().run_until_complete(self._dumb_prompt(self.message))
+            with self._dumb_prompt(self.message) as dump_app:
+                return dump_app.run(in_thread=in_thread)
 
-        return self.app.run(set_exception_handler=set_exception_handler)
+        return self.app.run(
+            set_exception_handler=set_exception_handler, in_thread=in_thread
+        )
 
-    async def _dumb_prompt(self, message: AnyFormattedText = "") -> _T:
+    @contextmanager
+    def _dumb_prompt(self, message: AnyFormattedText = "") -> Iterator[Application[_T]]:
         """
-        Prompt function for dumb terminals.
+        Create prompt `Application` for prompt function for dumb terminals.
 
         Dumb terminals have minimum rendering capabilities. We can only print
         text to the screen. We can't use colors, and we can't do cursor
@@ -1050,13 +1060,15 @@ class PromptSession(Generic[_T]):
             self.output.flush()
 
         self.default_buffer.on_text_changed += on_text_changed
-        result = await application.run_async()
 
-        # Render line ending.
-        self.output.write("\r\n")
-        self.output.flush()
+        try:
+            yield application
+        finally:
+            # Render line ending.
+            self.output.write("\r\n")
+            self.output.flush()
 
-        return result
+            self.default_buffer.on_text_changed -= on_text_changed
 
     async def prompt_async(
         self,
@@ -1190,7 +1202,8 @@ class PromptSession(Generic[_T]):
         # If we are using the default output, and have a dumb terminal. Use the
         # dumb prompt.
         if self._output is None and is_dumb_terminal():
-            return await self._dumb_prompt(self.message)
+            with self._dumb_prompt(self.message) as dump_app:
+                return await dump_app.run_async()
 
         return await self.app.run_async(set_exception_handler=set_exception_handler)
 
@@ -1292,7 +1305,7 @@ class PromptSession(Generic[_T]):
         return self._get_continuation(prompt_width, line_number, wrap_count)
 
     def _get_arg_text(self) -> StyleAndTextTuples:
-        " 'arg' toolbar, for in multiline mode. "
+        "'arg' toolbar, for in multiline mode."
         arg = self.app.key_processor.arg
         if arg is None:
             # Should not happen because of the `has_arg` filter in the layout.
@@ -1304,7 +1317,7 @@ class PromptSession(Generic[_T]):
         return [("class:arg-toolbar", "Repeat: "), ("class:arg-toolbar.text", arg)]
 
     def _inline_arg(self) -> StyleAndTextTuples:
-        " 'arg' prefix, for in single line mode. "
+        "'arg' prefix, for in single line mode."
         app = get_app()
         if app.key_processor.arg is None:
             return []
@@ -1449,7 +1462,7 @@ def create_confirm_session(
 
     @bindings.add(Keys.Any)
     def _(event: E) -> None:
-        " Disallow inserting other text. "
+        "Disallow inserting other text."
         pass
 
     complete_message = merge_formatted_text([message, suffix])
